
#include "lfo.h"
#include <lunar/fx.hpp>

class lfo : public lunar::fx<lfo> {
public:
	float phase;
	float val;
	float amp;
	float freq;
	float minval;
	int intervaltype;
	int intervalsize;

	void init() {
		val = 0;
		phase = 0;
	}

	void exit() {
		delete this;
	}

	void process_timer(int* numsamples, int* processevents) {
		phase = 2 * M_PI * ((float)transport->song_position / transport->samples_per_second);
		val = ((1.0f+sin(phase * freq))*0.5f) * amp + minval;
		val = min(max(val, 0), 1);

		switch (intervaltype) {
			case 0:
				*numsamples = (int)(transport->samples_per_tick * intervalsize);
				break;
			case 1:
				*numsamples = (int)(transport->samples_per_tick * intervalsize / 256);
				break;
			case 2:
				*numsamples = (int)((float)transport->samples_per_second / 1000 * intervalsize);
				break;
		}
		*processevents = 1;
	}

	void process_events() {
		if (globals->intervaltype)
			intervaltype = *globals->intervaltype;
		if (globals->intervalsize)
			intervalsize = *globals->intervalsize;
		if (globals->freq)
			freq = *globals->freq;
		if (globals->amp)
			amp = *globals->amp;
		if (globals->minval)
			minval = *globals->minval;
	}
	
	void process_controller_events() {
		controllers->out1 = &val;
	}
};

lunar_fx *new_fx() { return new lfo(); }
